const Soup = imports.gi.Soup;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;

const ExtensionSystem = imports.ui.extensionSystem;
const ExtensionUtils = imports.misc.extensionUtils;

const Extension = ExtensionUtils.getCurrentExtension();

// ----------------------------------------------------------------------------------------


const _cache = {};

const cacheDir = Gio.file_new_for_path(GLib.get_user_cache_dir()).get_child(Extension.uuid);
const cacheFile = cacheDir.get_child('extensions.json');

const CACHE_EXPIRE = 86400;

function init() {
	if(!cacheDir.query_exists(null)) {
		cacheDir.make_directory_with_parents(null);
	}
	if(cacheFile.query_exists(null)) {
		try {
			[ succ, data, tag ] = cacheFile.load_contents(null);
			if(succ) {
				_cache = JSON.parse(data);
			}
		} catch(e) {
			logError(e, "Couldn't load cache.");
		}
	}
}

function uninit() {
	try {
		[ succ ] = cacheFile.replace_contents(
				JSON.stringify(_cache), null, false, Gio.FileCreateFlags.NONE, null);
		if(!succ)
			logError(new Error('Unable to save data.'));
	} catch(e) {
		logError(e, "Couldn't save cache.");
	}
}

// XXX: Should I be using the ExtensionSystem's _httpSession or starting my own?
function _getSoup() {
	return ExtensionSystem._httpSession;
}

function lookup(uuid, onSuccess, onFailure) {
	if(uuid in _cache) {
		if(((new Date().getTime() / 1000) - _cache[uuid]._cache_time) < CACHE_EXPIRE) {
			onSuccess(_cache[uuid]);
			return;
		}
	}

	delete _cache[uuid];

	let msg = Soup.form_request_new_from_hash('GET', ExtensionSystem.REPOSITORY_URL_INFO, {
		uuid: uuid
	});

	_getSoup().queue_message(msg, function(soup, msg) {
		if(msg.status_code != Soup.KnownStatusCode.OK) {
			onFailure();
		} else {
			let data = JSON.parse(msg.response_body.data);
			data._cache_time = new Date().getTime();
			_cache[data.uuid] = data;
			onSuccess(data);
		}
	});
}

function sendPopularity(uuid, action) {
	let msg = Soup.form_request_new_from_hash('POST', ExtensionSystem.REPOSITORY_URL_BASE + '/ajax/adjust-popularity', {
		uuid: uuid,
		action: action
	});

	_getSoup().queue_message(msg, function() {
		// Nothing to see here.
	});
}
